@@ -1,7 +1,8 @@
 from ultron.utilities.logger import kd_logger
 from ultron.factor.data.processing import factor_processing
 from ultron.factor.data.winsorize import winsorize_normal
-from ultron.tradingday import *
+#from ultron.tradingday import *
+from PyFin.api import *
 import numpy as np
 import pandas as pd
 import copy,pdb,math
@@ -37,6 +38,23 @@ class MultipleFactor(object):
         mixed_signal = mixed_signal.dropna(subset=['signal'])
         return mixed_signal
 
+    def _transformer(self, signal):
+        s = copy.deepcopy(signal).sort_values(by=['signal'])
+        equal = s[s['signal'] == 0]
+        nequal = s[s['signal'] != 0]
+        le = len(nequal)
+        short = nequal[:int(le/2)]
+        long = nequal[int(le/2):]
+        short['signal'] = short['signal'].apply(lambda x: x if x < 0 else  0-x)
+        long['signal'] = long['signal'].apply(lambda x: x if x > 0 else 0-x)
+        return pd.concat([long,short,equal],axis=0).sort_values(
+            by=['signal']).reset_index(drop=True)
+
+    def _symmetry(self, mixed_signal):
+        signal = mixed_signal.set_index('trade_date').groupby(
+            level=['trade_date']).apply(lambda x: self._transformer(x))
+        return signal.reset_index().drop(['level_1'],axis=1)
+
     def _initialize_weight(self, mixed_signal, volatility_name):
         data = mixed_signal.merge(
             self._volatility_data, on=['trade_date','code'])
@@ -89,12 +107,30 @@ class MultipleFactor(object):
         return pd.concat(alpha_res).sort_values(by=['trade_date','code'])
 
 
+    def _weighted_normalised(self, target_pos):
+        long_target = target_pos[target_pos['weight'] > 0]
+        short_target = target_pos[target_pos['weight'] < 0]
+        equal_target = target_pos[target_pos['weight'] == 0]
+        long_target = long_target.reset_index().set_index(['trade_date']).groupby(
+            level=['trade_date']).apply(lambda x: self._weighted(data=x, equal=0, name='weight')
+            ).reset_index()[['trade_date','weight','code']]
+        short_target = short_target.reset_index().set_index(['trade_date']).groupby(
+            level=['trade_date']).apply(lambda x: self._weighted(data=x, equal=0, name='weight')
+            ).reset_index()[['trade_date','weight','code']]
+        short_target['weight'] = 0 - short_target['weight']
+        new_target_pos = long_target.append(short_target).append(equal_target.reset_index())
+        return new_target_pos.sort_values(by=['trade_date','code']).set_index(['trade_date','code'])
+
     ### 权重计算
-    def weighted_run(self, columns=None, default_volatility=0.1, volatility_name='STD20D', period=20):
+    def weighted_run(self, columns=None, default_volatility=0.1, volatility_name='STD20D', period=20, 
+                    is_symmetry=True, is_normalised=True):
         kd_logger.info("starting construction")
         columns = columns if columns is not None else [col for col in self._signal_data.columns if col \
             not in ['trade_date','code']]
         mixed_signal = self._mixed_signal(columns)
+        if is_symmetry:
+            mixed_signal = self._symmetry(mixed_signal)
+            
         kd_logger.info("initialize weight")
 
         weighted = self._initialize_weight(mixed_signal, volatility_name)
@@ -151,16 +187,18 @@ class MultipleFactor(object):
                 weighted_data = copy.deepcopy(this_data)
                 weighted_data['weight'] = ((default_volatility / op) * this_data['weight'])
                 res.append(weighted_data.set_index(['trade_date','code']))
-        return res
+        target_pos = pd.concat(res,axis=0)
+        return target_pos if not is_normalised else self._weighted_normalised(target_pos)
+
+
     
-
-
-
-    def equal_run(self, columns=None, equal=1):
+    def equal_run(self, columns=None, equal=1,is_symmetry=True):
         kd_logger.info("starting construction")
         columns = columns if columns is not None else [col for col in self._signal_data.columns if col \
             not in ['trade_date','code']]
         mixed_signal = self._mixed_signal(columns)
+        if is_symmetry:
+            mixed_signal = self._symmetry(mixed_signal)
         ###权重换算
         long_signal = mixed_signal[mixed_signal.signal > 0]
         short_signal = mixed_signal[mixed_signal.signal < 0]
